/*********************
 *      INCLUDES
 *********************/
#if defined(LV_LVGL_H_INCLUDE_SIMPLE)
#include "lvgl.h"
#else
#include "../lvgl/lvgl.h"
#endif

#include <stdio.h>
#include "a/ugui.h"
#include "a/consola_67X121.h"

#define DBG printf("Here! Line %d\n", __LINE__)

typedef signed long pos;
typedef struct { pos x, y; } vec;
typedef struct { pos xx, xy, yx, yy; } matrix;

#define W 67
#define H 121
UG_COLOR my_img_data[W*H] = {0};
UG_COLOR my_img_data2[W*H*2] = {0};
static lv_img_dsc_t my_img_dsc = {
    .header.always_zero = 0,
    .header.w = W,
    .header.h = H,
    .data_size = W*H*sizeof(UG_COLOR),
    .header.cf = LV_IMG_CF_TRUE_COLOR_ALPHA,          /*Set the color format*/
    .data = (const uint8_t*) my_img_data,
};
static lv_img_dsc_t my_img_dsc2 = {
    .header.always_zero = 0,
    .header.w = W*2,
    .header.h = H,
    .data_size = W*2*H*sizeof(UG_COLOR),
    .header.cf = LV_IMG_CF_TRUE_COLOR_ALPHA,          /*Set the color format*/
    .data = (const uint8_t*) my_img_data2,
};

static lv_img_dsc_t * img;
static void drawPixel(UG_S16 x, UG_S16 y, UG_COLOR col)
{
	UG_COLOR* pixel = (UG_COLOR*) &img->data[(W*y + x)*sizeof(UG_COLOR)];
	*pixel = col | (0xFF << 24);
}

static void showFont(const UG_FONT * font, char* text, lv_img_dsc_t *target_img)
{
  static UG_GUI gui;
  img = target_img;
  UG_Init(&gui, drawPixel, W, H);

  UG_FillScreen(C_WHITE);
  UG_FontSelect(font);
  UG_SetBackcolor(C_WHITE);
  UG_SetForecolor(C_BLACK);
  UG_PutString(0, 0, text);

  UG_Update();
  printf("\n");
}

signed long mulfix(signed long a, signed long b) {
	return (a*b)/0x10000;
}

void vector_transform( vec* v, matrix* m) {
    pos xz, yz;

    if ( !v || !m )
      return;

    xz = mulfix( v->x, m->xx ) +
         mulfix( v->y, m->xy );

    yz = mulfix( v->x, m->yx ) +
         mulfix( v->y, m->yy );

    v->x = xz;
    v->y = yz;
}

void make_oblique(void) {
	matrix m;
	m.xx = 0x10000L;
    m.xy = -0x5000L;
    m.yx = 0;
    m.yy = 0x10000L;
    
    vec src, target;
    
    for (int x=0; x<W; x++) for (int y=0; y<H; y++) {
		src.x = target.x = x;
		src.y = target.y = y;
		vector_transform(&target, &m);
		my_img_data2[target.y*W*2 + target.x + W/2] = my_img_data[src.y*W + src.x];
		printf("moving %dx%d to %dx%d\n", src.x, src.y, target.x, target.y);
	}
}

void lv_oblique_test(void) {
	showFont(&font_consola_67X121, "B", &my_img_dsc);
	//showFont(&font_consola_67X121, "A", &my_img_dsc2);
	make_oblique();
	lv_obj_t * icon = lv_img_create(lv_scr_act(), NULL);
	lv_obj_t * icon2 = lv_img_create(lv_scr_act(), NULL);
	lv_obj_set_pos(icon2, 68, 0);

    /*From variable*/
    lv_img_set_src(icon, &my_img_dsc);
    lv_img_set_src(icon2, &my_img_dsc2);
}
